<?php

namespace Drupal\taxonomy_scheduler\Service;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\taxonomy_scheduler\ValueObject\TaxonomyFieldStorageItem;
use Psr\Log\LoggerInterface;

/**
 * Class TaxonomySchedulerField.
 */
class TaxonomySchedulerField {

  use StringTranslationTrait;

  /**
   * Messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  private $messenger;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  private $logger;

  /**
   * FieldConfig.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  private $fieldConfig;

  /**
   * FieldStorageConfig.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  private $fieldStorageConfig;

  /**
   * EntityFormDisplay.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  private $entityFormDisplay;

  /**
   * TaxonomySchedulerField constructor.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   * @param \Drupal\Core\Entity\EntityStorageInterface $fieldStorageConfig
   *   The field storage config.
   * @param \Drupal\Core\Entity\EntityStorageInterface $fieldConfig
   *   The field config.
   * @param \Drupal\Core\Entity\EntityStorageInterface $entityFormDisplay
   *   The entity form display.
   */
  public function __construct(
    MessengerInterface $messenger,
    LoggerInterface $logger,
    EntityStorageInterface $fieldStorageConfig,
    EntityStorageInterface $fieldConfig,
    EntityStorageInterface $entityFormDisplay
  ) {
    $this->messenger = $messenger;
    $this->logger = $logger;
    $this->fieldStorageConfig = $fieldStorageConfig;
    $this->fieldConfig = $fieldConfig;
    $this->entityFormDisplay = $entityFormDisplay;
  }

  /**
   * Adds the field configurations.
   *
   * @param \Drupal\taxonomy_scheduler\ValueObject\TaxonomyFieldStorageItem $fieldStorageItem
   *   The field storage item value object.
   */
  public function addFieldConfigurations(TaxonomyFieldStorageItem $fieldStorageItem): void {
    $fieldName = $fieldStorageItem->getFieldName();
    $vocabularies = $fieldStorageItem->getVocabularies();
    $fieldLabel = $fieldStorageItem->getFieldLabel();
    $fieldRequired = $fieldStorageItem->getFieldRequired();

    $existingField = $this->fieldConfig->load('taxonomy_term.' . $fieldName);

    if ($existingField !== NULL) {
      $this->messenger->addMessage($this->t('A field by name :field already exists.', [':field' => $fieldName]));
      return;
    }

    foreach ($vocabularies as $vocabulary) {
      try {
        $fieldStorageConfig = [
          'entity_type' => 'taxonomy_term',
          'field_name' => $fieldName,
          'type' => 'datetime',
          'settings' => [],
          'cardinality' => 1,
        ];

        $fieldConfig = [
          'entity_type' => 'taxonomy_term',
          'field_name' => $fieldName,
          'bundle' => $vocabulary,
          'label' => $fieldLabel,
          'required' => $fieldRequired,
        ];

        $this->fieldStorageConfig->create($fieldStorageConfig)->save();
        $this->fieldConfig->create($fieldConfig)->save();
        $this->setFormDisplay($vocabulary, $fieldName);
      }
      catch (EntityStorageException $e) {
        $this->logger->error($e->getMessage());
      }
    }
  }

  /**
   * Sets the form display.
   *
   * @param string $bundle
   *   The bundle.
   * @param string $fieldName
   *   The field name.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function setFormDisplay(string $bundle, string $fieldName): void {
    /** @var \Drupal\Core\Entity\Display\EntityFormDisplayInterface $formDisplay */
    $formDisplay = $this->entityFormDisplay->load('taxonomy_term.' . $bundle . '.default');

    if(!$formDisplay) {
      $formDisplay = $this->entityFormDisplay->create([
        'targetEntityType' => 'taxonomy_term',
        'bundle' => $bundle,
        'mode' => 'default',
        'status' => TRUE
      ]);
      $formDisplay->save();
    }

    if (!$formDisplay instanceof EntityInterface) {
      return;
    }

    $formDisplay->setComponent($fieldName, ['type' => 'datetime_default'])
      ->save();
  }

}
